@@ -3,6 +3,7 @@
 
 open Lexing
 type t = { loc_start: position; loc_end: position; loc_ghost: bool }
+type 'a loc = { txt: 'a; loc: t }
 
 let none = { loc_start = dummy_pos; loc_end = dummy_pos; loc_ghost = true }
 let dummy x = assert false
@@ -23,3 +24,9 @@ let prerr_warning = dummy
 let echo_eof = dummy
 let reset = dummy
 let highlight_locations = dummy
+let mknoloc = dummy
+let mkloc = dummy
+let print_loc = dummy
+let print_filename = dummy
+let show_filename = dummy
+let absname = ref true
