@@ -1,9 +1,3 @@
-(**************************************************************************)
-(*  The CDuce compiler                                                    *)
-(*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003-2009 (see LICENSE for details)             *)
-(**************************************************************************)
-
 #load "q_MLast.cmo";;
 (* TODO:
    - optimizations: generate labels and atoms only once.
@@ -53,6 +47,13 @@ let consId s =
   in
   aux 0
 
+let ident_to_string list =
+  let rec _ident_to_string list res = match list with
+    | (id, x) :: rest -> _ident_to_string rest (res @ [id.Caml_cduce.Ident.name, x])
+    | [] -> res
+  in
+  _ident_to_string list [];;
+
 let rec typ t =
   try IntHash.find memo_typ t.uid
   with Not_found ->
@@ -68,9 +69,9 @@ and typ_descr = function
   | Tuple tl -> Types.tuple (List.map typ tl)
   | PVariant l -> bigcup pvariant l
   | Variant (_,l,_) -> bigcup variant l
-  | Record (_,l,_) ->
-      let l = List.map (fun (lab,t) -> label lab, typ t) l in
-      Types.record_fields (false,(LabelMap.from_list_disj l))
+  | Record (_,l,_) -> let l = ident_to_string l in
+		      let l = List.map (fun (lab,t) -> label lab, typ t) l in
+		      Types.record_fields (false, (LabelMap.from_list_disj l))
   | Abstract "int" -> Builtin_defs.caml_int
   | Abstract "char" -> Builtin_defs.char_latin1
   | Abstract "string" -> Builtin_defs.string_latin1
@@ -92,8 +93,10 @@ and pvariant = function
   | (lab, Some t) -> Types.times (Types.cons (atom lab)) (typ t)
 
 and variant = function
-  | (lab, []) -> atom lab
-  | (lab, c) -> Types.tuple (Types.cons (atom lab) :: List.map typ c)
+  | (lab, [], None) -> atom lab.Caml_cduce.Ident.name
+  | (lab, [], Some o) -> Types.tuple (Types.cons (atom lab.Caml_cduce.Ident.name) :: List.map typ [o])
+  | (lab, c, Some o) -> Types.tuple (Types.cons (atom lab.Caml_cduce.Ident.name) :: List.map typ (c@[o]))
+  | (lab, c, None) -> Types.tuple (Types.cons (atom lab.Caml_cduce.Ident.name) :: List.map typ c)
 
 
 (* Syntactic tools *)
@@ -224,70 +227,74 @@ and to_cd_descr e = function
   | Link t -> to_cd e t
   | Arrow (l,t,s) -> 
       (* let y = <...> in Value.Abstraction ([t,s], fun x -> s(y ~l:(t(x))) *)
-      protect e 
+    protect e 
       (fun y ->
-	 let x = mk_var () in
-	 let arg = to_ml <:expr< $lid:x$ >> t in
-	 let res = to_cd (call_lab y l arg) s in
-	 let abs = <:expr< fun $lid:x$ -> $res$ >> in
-	 let iface =
-	   if !gen_types then
-	     let tt = register_type (Types.descr (typ t)) in
-	     let ss = register_type (Types.descr (typ s)) in
-	     <:expr< Some [($tt$,$ss$)] >>
-	   else <:expr< None >> in
-	 <:expr< Value.Abstraction ($iface$,$abs$) >>
+	let x = mk_var () in
+	let arg = to_ml <:expr< $lid:x$ >> t in
+	let res = to_cd (call_lab y l arg) s in
+	let abs = <:expr< fun $lid:x$ -> $res$ >> in
+	let iface =
+	  if !gen_types then
+	    let tt = register_type (Types.descr (typ t)) in
+	    let ss = register_type (Types.descr (typ s)) in
+	    <:expr< Some [($tt$,$ss$)] >>
+	  else <:expr< None >> in
+	<:expr< Value.Abstraction ($iface$,$abs$) >>
       )
   | Tuple tl -> 
       (* let (x1,...,xn) = ... in Value.Pair (t1(x1), Value.Pair(...,tn(xn))) *)
-      let vars = mk_vars tl in
-      <:expr< let $pat_tuple vars$ = $e$ in $tuple (tuple_to_cd tl vars)$ >>
+    let vars = mk_vars tl in
+    <:expr< let $pat_tuple vars$ = $e$ in $tuple (tuple_to_cd tl vars)$ >>
   | PVariant l ->
       (* match <...> with 
 	 | `A -> Value.atom_ascii "A" 
 	 | `B x -> Value.Pair (Value.atom_ascii "B",t(x))
       *)
-      let cases = 
-	List.map
-	  (function 
-	     | (lab,None) -> <:match_case< `$lid:lab$ -> $atom_ascii lab$ >>
-	     | (lab,Some t) -> <:match_case< `$lid:lab$ x ->
-		 $pair (atom_ascii lab) (to_cd <:expr< x >> t)$ >>
-	  ) l in
-      pmatch e cases
+    let cases = 
+      List.map
+	(function 
+	  | (lab,None) -> <:match_case< `$lid:lab$ -> $atom_ascii lab$ >>
+	  | (lab,Some t) -> <:match_case< `$lid:lab$ x ->
+	    $pair (atom_ascii lab) (to_cd <:expr< x >> t)$ >>
+	) l in
+    pmatch e cases
   | Variant (p,l,_) ->
       (* match <...> with 
 	 | P.A -> Value.atom_ascii "A" 
 	 | P.B (x1,x2,..) -> Value.Pair (Value.atom_ascii "B",...,Value.Pair(tn(x)))
       *)
-      let cases = 
-	List.map
-	  (function 
-	     | (lab,[]) ->
-		 let pat = match lab with (* Stupid Camlp4 *)
-		    | "true" -> <:patt< True >>
-		    | "false" -> <:patt< False >>
-		    | lab -> <:patt< $id: id (p^lab)$ >>
-		 in
-		 <:match_case< $pat$ -> $atom_ascii lab$ >>
-	     | (lab,tl) -> 
-		 let vars = mk_vars tl in
-		 <:match_case< $id: id (p^lab)$ $pat_tuple vars$ ->
-	           $tuple (atom_ascii lab :: tuple_to_cd tl vars)$ >>
-	  ) l in
-      pmatch e cases
+    let cases = 
+      List.map
+	(function 
+	  | (lab,[],None) ->
+	    let pat = match lab.Caml_cduce.Ident.name with (* Stupid Camlp4 *)
+	      | "true" -> <:patt< True >>
+	      | "false" -> <:patt< False >>
+	      | lab -> <:patt< $id: id (p^lab)$ >>
+	    in
+            <:match_case< $pat$ -> $atom_ascii lab.Caml_cduce.Ident.name$ >>
+	  | (lab,tl,Some o) ->
+            let vars = mk_vars (tl@[o]) in
+            <:match_case< $id: id (p^(lab.Caml_cduce.Ident.name))$ $pat_tuple vars$ ->
+            $tuple (atom_ascii lab.Caml_cduce.Ident.name :: tuple_to_cd (tl@[o]) vars)$ >>
+	  | (lab,tl,None) ->
+            let vars = mk_vars tl in
+            <:match_case< $id: id (p^(lab.Caml_cduce.Ident.name))$ $pat_tuple vars$ ->
+            $tuple (atom_ascii lab.Caml_cduce.Ident.name :: tuple_to_cd tl vars)$ >>
+        ) l in
+    pmatch e cases
   | Record (p,l,_) ->
       (* let x = <...> in Value.record [ l1,t1(x.P.l1); ...; ln,x.P.ln ] *)
-      protect e
+    protect e
       (fun x ->
-	 let l = 
-	   List.map
-	     (fun (lab,t) ->
-		let e = to_cd <:expr<$x$.$id:id (p^lab)$>> t in
-		<:expr< ($label_ascii lab$, $e$) >>)
-	     l
-	 in
-	 <:expr< Value.record $list_lit l$ >>)
+	let l = 
+	  List.map
+	    (fun (lab,t) ->
+	      let lab = lab.Caml_cduce.Ident.name in
+	      let e = to_cd <:expr<$x$.$id:id (p^lab)$>> t in
+	      <:expr< ($label_ascii lab$, $e$) >>) l
+       in
+       <:expr< Value.record $list_lit l$ >>)
 
   | Abstract "int" -> <:expr< Value.ocaml2cduce_int $e$ >>
   | Abstract "char" -> <:expr< Value.ocaml2cduce_char $e$ >>
@@ -393,19 +400,41 @@ and to_ml_descr e = function
       let cases = 
 	List.map 
 	  (function 
-	     | (lab,[]) -> 
+	     | (lab,[],None) -> 
+	       let lab = lab.Caml_cduce.Ident.name in
 		 let pa = <:patt< ($str: String.escaped lab$, None) >>
 		 and e = match lab with (* Stupid Camlp4 *)
 		    | "true" -> <:expr< True >>
 		    | "false" -> <:expr< False >>
 		    | lab -> <:expr< $id:id (p ^ lab)$ >> in
 		 <:match_case< $pa$ -> $e$ >>
-	     | (lab,[t]) ->
+	     | (lab,[t],None) ->
+	       let lab = lab.Caml_cduce.Ident.name in
 		 let x = mk_var () in
 		 let ex = <:expr< $lid:x$ >> in
 		 <:match_case< ($str: String.escaped lab$, Some $lid:x$) ->
 		  $id:id (p ^ lab)$ $to_ml ex t$ >>
-	     | (lab,tl) ->
+	     | (lab,[],Some o) ->
+	       let lab = lab.Caml_cduce.Ident.name in
+		 let x = mk_var () in
+		 let ex = <:expr< $lid:x$ >> in
+		 <:match_case< ($str: String.escaped lab$, Some $lid:x$) ->
+		  $id:id (p ^ lab)$ $to_ml ex o$ >>
+	     | (lab,tl,Some o) ->
+	       let lab = lab.Caml_cduce.Ident.name in
+		 let vars = mk_vars (tl@[o]) in
+		 let x = mk_var () in
+		 <:match_case< ($str: String.escaped lab$, Some $lid:x$) ->
+   	       $ matches
+               <:expr< $lid:x$ >> (
+                      List.fold_left
+                        (fun x (t, id) ->
+                          Ast.ExApp(_loc, x, <:expr<$to_ml <:expr< $lid:id$ >> t$>>))
+                 <:expr< $id:consId (p ^ lab)$ >>
+                        (List.combine (tl@[o]) vars))
+                 vars $ >>
+	     | (lab,tl,None) ->
+	       let lab = lab.Caml_cduce.Ident.name in
 		 let vars = mk_vars tl in
 		 let x = mk_var () in
 		 <:match_case< ($str: String.escaped lab$, Some $lid:x$) ->
@@ -430,6 +459,7 @@ and to_ml_descr e = function
 	 let l = 
 	   List.map
 	     (fun (lab,t) ->
+	       let lab = lab.Caml_cduce.Ident.name in
 		let e = 
 		  to_ml <:expr< Value.get_field $x$ $label_ascii lab$ >> t in
 		<:rec_binding< $id: id (p^lab)$ = $e$ >>) l in
