@@ -1,9 +1,3 @@
-(**************************************************************************)
-(*  The CDuce compiler                                                    *)
-(*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003-2009 (see LICENSE for details)             *)
-(**************************************************************************)
-
 open Caml_cduce
 open Asttypes
 open Types
@@ -16,13 +10,12 @@ and def =
   | Arrow of string * t * t
   | Tuple of t list
   | PVariant of (string * t option) list  (* Polymorphic variant *)
-  | Variant of string * (string * t list) list * bool
-  | Record of string * (string * t) list * bool
+  | Variant of string * (Ident.t * t list * t option) list * bool
+  | Record of string * (Ident.t * t) list * bool
   | Builtin of string * t list
   | Abstract of string
   | Var of int
 
-
 val reg_uid: t -> unit
 
 (* Load an external .cmi *)
