@@ -85,7 +85,7 @@ let nb_classes = 40
 
   let in_comment = ref false
 
-  let error = Location.raise_loc
+  let error = Cduce_loc.raise_loc
   exception Illegal_character of char
   exception Unterminated_comment
   exception Unterminated_string
@@ -314,7 +314,7 @@ and comment engine lexbuf =
 	Buffer.clear string_buff;
 	let ender = Lexing.lexeme lexbuf in
         (try string ender engine lexbuf
-         with Location.Location (_,_,Unterminated_string) ->
+         with Cduce_loc.Location (_,_,Unterminated_string) ->
            let st = List.hd !comment_start_pos in
            error st (st+2) Unterminated_string_in_comment);
 	Buffer.clear string_buff;
